@@ -23,10 +23,9 @@
 #define AVCODEC_FAANIDCT_H
 
 #include <stdint.h>
-#include "dsputil.h"
 
-void ff_faanidct(DCTELEM block[64]);
-void ff_faanidct_add(uint8_t *dest, int line_size, DCTELEM block[64]);
-void ff_faanidct_put(uint8_t *dest, int line_size, DCTELEM block[64]);
+void ff_faanidct(int16_t block[64]);
+void ff_faanidct_add(uint8_t *dest, int line_size, int16_t block[64]);
+void ff_faanidct_put(uint8_t *dest, int line_size, int16_t block[64]);
 
 #endif /* AVCODEC_FAANIDCT_H */
