@@ -30,14 +30,11 @@
 typedef struct ProresDSPContext {
     int idct_permutation_type;
     uint8_t idct_permutation[64];
-    int dct_permutation_type;
-    uint8_t dct_permutation[64];
     void (* idct_put) (uint16_t *out, int linesize, int16_t *block, const int16_t *qmat);
-    void (* fdct) (const uint16_t *src, int linesize, int16_t *block);
 } ProresDSPContext;
 
 void ff_proresdsp_init(ProresDSPContext *dsp);
 
-void ff_proresdsp_x86_init(ProresDSPContext *dsp);
+void ff_proresdsp_init_x86(ProresDSPContext *dsp);
 
 #endif /* AVCODEC_PRORESDSP_H */
