@@ -18,6 +18,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#include "libavutil/attributes.h"
 #include "libavutil/common.h"
 #include "videodsp.h"
 
@@ -33,7 +34,7 @@ static void just_return(uint8_t *buf, ptrdiff_t stride, int h)
 {
 }
 
-void ff_videodsp_init(VideoDSPContext *ctx, int bpc)
+av_cold void ff_videodsp_init(VideoDSPContext *ctx, int bpc)
 {
     ctx->prefetch = just_return;
     if (bpc <= 8) {
@@ -42,6 +43,8 @@ void ff_videodsp_init(VideoDSPContext *ctx, int bpc)
         ctx->emulated_edge_mc = ff_emulated_edge_mc_16;
     }
 
+    if (ARCH_AARCH64)
+        ff_videodsp_init_aarch64(ctx, bpc);
     if (ARCH_ARM)
         ff_videodsp_init_arm(ctx, bpc);
     if (ARCH_PPC)
