@@ -1,5 +1,5 @@
 /*
- * MMX optimized forward DCT
+ * SIMD-optimized forward DCT
  * The gcc porting is Copyright (c) 2001 Fabrice Bellard.
  * cleanup/optimizations are Copyright (c) 2002-2004 Michael Niedermayer <michaelni@gmx.at>
  * SSE2 optimization is Copyright (c) 2004 Denes Balatoni.
@@ -32,9 +32,9 @@
 
 #include "libavutil/common.h"
 #include "libavutil/x86/asm.h"
-#include "libavcodec/dsputil.h"
+#include "fdct.h"
 
-#if HAVE_INLINE_ASM
+#if HAVE_MMX_INLINE
 
 //////////////////////////////////////////////////////////////////////
 //
@@ -556,6 +556,10 @@ void ff_fdct_mmx(int16_t *block)
     }
 }
 
+#endif /* HAVE_MMX_INLINE */
+
+#if HAVE_MMXEXT_INLINE
+
 void ff_fdct_mmxext(int16_t *block)
 {
     DECLARE_ALIGNED(8, int64_t, align_tmp)[16];
@@ -574,6 +578,10 @@ void ff_fdct_mmxext(int16_t *block)
     }
 }
 
+#endif /* HAVE_MMXEXT_INLINE */
+
+#if HAVE_SSE2_INLINE
+
 void ff_fdct_sse2(int16_t *block)
 {
     DECLARE_ALIGNED(16, int64_t, align_tmp)[16];
@@ -583,4 +591,4 @@ void ff_fdct_sse2(int16_t *block)
     fdct_row_sse2(block1, block);
 }
 
-#endif /* HAVE_INLINE_ASM */
+#endif /* HAVE_SSE2_INLINE */
