@@ -77,6 +77,10 @@ const AVCodecTag ff_nut_video_tags[] = {
     { AV_CODEC_ID_RAWVIDEO,         MKTAG('R', 'G', 'B',  48) },
     { AV_CODEC_ID_RAWVIDEO,         MKTAG(48,  'B', 'G', 'R') },
     { AV_CODEC_ID_RAWVIDEO,         MKTAG(48,  'R', 'G', 'B') },
+    { AV_CODEC_ID_RAWVIDEO,         MKTAG('R', 'B', 'A', 64 ) },
+    { AV_CODEC_ID_RAWVIDEO,         MKTAG('B', 'R', 'A', 64 ) },
+    { AV_CODEC_ID_RAWVIDEO,         MKTAG(64 , 'R', 'B', 'A') },
+    { AV_CODEC_ID_RAWVIDEO,         MKTAG(64 , 'B', 'R', 'A') },
     { AV_CODEC_ID_RAWVIDEO,         MKTAG('Y', '3',  11,  10) },
     { AV_CODEC_ID_RAWVIDEO,         MKTAG(10,   11, '3', 'Y') },
     { AV_CODEC_ID_RAWVIDEO,         MKTAG('Y', '3',  10,  10) },
