@@ -22,9 +22,6 @@
 #ifndef AVFORMAT_NUT_H
 #define AVFORMAT_NUT_H
 
-//#include <limits.h>
-//#include "libavutil/adler32.h"
-//#include "libavcodec/mpegaudio.h"
 #include "avformat.h"
 #include "internal.h"
 #include "metadata.h"
@@ -39,6 +36,10 @@
 
 #define MAX_DISTANCE (1024*32-1)
 
+#define NUT_MAX_VERSION 4
+#define NUT_STABLE_VERSION 3
+#define NUT_MIN_VERSION 2
+
 typedef enum{
     FLAG_KEY        =   1, ///<if set, frame is keyframe
     FLAG_EOR        =   2, ///<if set, stream has no relevance on presentation. (EOR)
@@ -86,6 +87,7 @@ typedef struct ChapterContext {
 } ChapterContext;
 
 typedef struct NUTContext {
+    const AVClass *av_class;
     AVFormatContext *avf;
 //    int written_packet_size;
 //    int64_t packet_start;
@@ -101,6 +103,10 @@ typedef struct NUTContext {
     int header_count;
     AVRational *time_base;
     struct AVTreeNode *syncpoints;
+#define NUT_BROADCAST 1 // use extended syncpoints
+#define NUT_PIPE 2      // do not write syncpoints
+    int flags;
+    int version; // version currently in use
 } NUTContext;
 
 extern const AVCodecTag ff_nut_subtitle_tags[];
